#!/usr/bin/perl
#
# cns2py.pl
#

$tempfile = "/tmp/perl.$$";

open(SORT_FH, "| sort > $tempfile") || die 'Cannot pipe to "sort".';

#
# This perl script converts the file cefdb of the KanjiBase for Windows
# package written by Christian Wittern into a form usable by the quail
# input system of MULE.
#
# character set:        CNS planes 3-7
# quail input system:   pinyin
#
# please note that not all CNS characters have a pinyin representation
# in the cefdb database. To access these missing characters use the
# 4 courner input method.
#
# Usage:    perl cns2py.pl < cefdb > py-CNS.el
#

$[ = 1;                 # set array base to 1
$FS = "\t";             # set field separator
$, = "\t";              # set output field separator
$\ = "\n";              # set output record separator


while (<>) {
    # We replace the SGML umlaut string with the MULE quail input system
    # equivalent.

    s/&ue;/u:/go;

    chop;               # strip record separator
    @Fld = split($FS, $_);

    # Now we check whether the current input line has more than eight
    # fields. If yes, remove the leading "Y" which marks a pinyin syllable,
    # print each syllable together with the CNS character code (the third
    # field) and sort it. All other fields of the database are ignored.
    #
    # Big 5 encoded characters (starting with C0-....) together with
    # characters from the reserved planes (CX- and CY-) will be left out.

    if ($#Fld >= 9) {
        $n = 9;
        while ($n <= $#Fld) {
            $i = index($Fld[$n], 'Y');
            if ($i != 0) {
                $plane = substr($Fld[3], 2, 1);
                print SORT_FH substr($Fld[$n], $i + 1), $Fld[3]
                  if (!($plane eq '0' || $plane eq 'X' || $plane eq 'Y'));
            }
            $n++;
        }
    }
}

close(SORT_FH);


# We now read the sorted file and arrange the entries as needed by the
# quail system: each pinyin syllable is followed by a string of CNS
# encoded characters.

open(TMP_FH, $tempfile);
$lastFld = '';
$outFld = '';


# The quail package header.

print "(require 'quail)";
print '(quail-define-package "py-CNS" "py-CNS"';
print ' t';
print ' "Chinese National Standard planes 3-7 pinyin input package';
print 'mechanically extracted from the cefdb database of the';
print 'KanjiBase for Windows package written by Christian Wittern."';
print " '(";
print '  (">" . quail-next-candidate-block)';
print '  ("<" . quail-prev-candidate-block)';
print '  (" " . quail-select-current)';
print '  )';
print ' nil nil)';
print '';
print ';;';


# The data section.

while (<TMP_FH>) {
    chop;
    @Fld = split($FS, $_);

    if ($Fld[1] eq $lastFld) {
        $outFld .= $Fld[2];
    }
    else {
        $outFld = &conv($outFld);
        $prefix = ($char_count == 1) ? 'qd  ' : 'qdv ';

        print '('.$prefix.'"'.$lastFld.'"', 
              '"'.$outFld.'")' if ($lastFld ne '');

        $lastFld = $Fld[1];
        $outFld = $Fld[2];
    }
}

$outFld = &conv($outFld);
$prefix = ($char_count == 1) ? 'qd  ' : 'qdv ';

print '('.$prefix.'"'.$lastFld.'"',         # the last line of data
      '"'.$outFld.'")' if ($lastFld ne '');


# The postamble.

print '';
print '(quail-setup-current-package)';

close(TMP_FH);
unlink $tempfile;

exit;


# The following subroutine converts the CEF representation of a CNS encoded
# character string into an ISO-2022 string as used by MULE.

sub conv {
    @CNS_plane_id = ('G' .. 'M');           # ISO 2022 final characters
    $last_CNS_plane = 0;
    $result = '';
    $char_count = 0;

    while (@_[1] =~ /C(.)-(..)(..)/go) {    # split CNS string char by char
        $char_count++;
        $result .= "\e".'$('."$CNS_plane_id[$1]" if ($1 != $last_CNS_plane);
        $result .= sprintf("%c%c", hex($2), hex($3));
        $last_CNS_plane = $1;
    }
    $result .= "\e(B";
    return $result;
}

# EOF
